@@ -1,6 +1,12 @@
 package test.baseline;
 
 import static aQute.bnd.osgi.Constants.BUNDLE_SYMBOLICNAME;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyMap;
 import static org.mockito.ArgumentMatchers.anyString;
@@ -15,6 +21,9 @@ import java.util.Set;
 import java.util.TreeSet;
 import java.util.regex.Pattern;
 
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Test;
+
 import aQute.bnd.build.Project;
 import aQute.bnd.build.ProjectBuilder;
 import aQute.bnd.build.Workspace;
@@ -32,18 +41,18 @@ import aQute.bnd.service.RepositoryPlugin;
 import aQute.bnd.service.diff.Delta;
 import aQute.bnd.service.diff.Diff;
 import aQute.bnd.service.diff.Tree;
+import aQute.bnd.test.jupiter.InjectTemporaryDirectory;
 import aQute.bnd.version.Version;
 import aQute.lib.collections.SortedList;
 import aQute.lib.io.IO;
 import aQute.libg.reporter.ReporterAdapter;
-import junit.framework.TestCase;
 
 @SuppressWarnings("resource")
-public class BaselineTest extends TestCase {
-	File		tmp;
+public class BaselineTest {
+
 	Workspace	workspace;
 
-	private Workspace getWorkspace() throws Exception {
+	private Workspace getWorkspace(File tmp) throws Exception {
 		if (workspace != null)
 			return workspace;
 
@@ -51,23 +60,16 @@ public class BaselineTest extends TestCase {
 		return workspace = new Workspace(tmp);
 	}
 
-	@Override
-	protected void setUp() throws Exception {
-		tmp = IO.getFile("generated/tmp/test/" + getClass().getName() + "/" + getName())
-			.getAbsoluteFile();
-		IO.delete(tmp);
-		IO.mkdirs(tmp);
-	}
-
-	@Override
+	@AfterEach
 	protected void tearDown() throws Exception {
-		IO.delete(tmp);
+		IO.close(workspace);
 		workspace = null;
 	}
 
 	/**
 	 * Test 2 jars compiled with different compilers
 	 */
+	@Test
 	public void testCompilerEnumDifference() throws Exception {
 		DiffPluginImpl diff = new DiffPluginImpl();
 		try (Jar ecj = new Jar(IO.getFile("jar/baseline/com.example.baseline.ecj.jar"));
@@ -83,6 +85,7 @@ public class BaselineTest extends TestCase {
 	/**
 	 * Test skipping classes when there is source
 	 */
+	@Test
 	public void testClassesDiffWithSource() throws Exception {
 		DiffPluginImpl diff = new DiffPluginImpl();
 		try (Jar jar = new Jar(IO.getFile("jar/osgi.jar")); Jar out = new Jar(".");) {
@@ -106,6 +109,7 @@ public class BaselineTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testClassesDiffWithoutSource() throws Exception {
 		DiffPluginImpl diff = new DiffPluginImpl();
 		try (Jar jar = new Jar(IO.getFile("jar/osgi.jar")); Jar out = new Jar(".");) {
@@ -125,6 +129,7 @@ public class BaselineTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testJava8DefaultMethods() throws Exception {
 		try (Builder older = new Builder(); Builder newer = new Builder();) {
 			older.addClasspath(IO.getFile("java8/older/bin"));
@@ -149,6 +154,7 @@ public class BaselineTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testNoMismatchForZeroMajor() throws Exception {
 		try (Builder older = new Builder(); Builder newer = new Builder();) {
 			older.addClasspath(IO.getFile("java8/older/bin"));
@@ -177,6 +183,7 @@ public class BaselineTest extends TestCase {
 	 * Check if we can ignore resources in the baseline. First build two jars
 	 * that are identical except for the b/b resource. Then do baseline on them.
 	 */
+	@Test
 	public void testIgnoreResourceDiff() throws Exception {
 		Processor processor = new Processor();
 		DiffPluginImpl differ = new DiffPluginImpl();
@@ -197,59 +204,13 @@ public class BaselineTest extends TestCase {
 		}
 	}
 
-	public static void testBaslineJar() throws Exception {
-		// Workspace ws = new Workspace(IO.getFile("testresources/ws"));
-		//
-		// Project p3 = ws.getProject("p3");
-		//
-		// ProjectBuilder builder = (ProjectBuilder)
-		// p3.getBuilder(null).getSubBuilder();
-		// builder.setBundleSymbolicName("p3");
-		//
-		// // Nothing specified
-		// Jar jar = builder.getBaselineJar(false);
-		// assertNull(jar);
-		//
-		// jar = builder.getBaselineJar(true);
-		// assertEquals(".", jar.getName());
-		//
-		// // Fallback to release repo
-		// builder.set("-releaserepo", "Repo");
-		// jar = builder.getBaselineJar(false);
-		// assertNull(jar);
-		//
-		// jar = builder.getBaselineJar(true);
-		// assertEquals("p3", jar.getBsn());
-		// assertEquals("1.0.1", jar.getVersion());
-		//
-		// // -baselinerepo specified
-		// builder.set("-baselinerepo", "Release");
-		// jar = builder.getBaselineJar(false);
-		// assertEquals("p3", jar.getBsn());
-		// assertEquals("1.2.0", jar.getVersion());
-		//
-		// jar = builder.getBaselineJar(true);
-		// assertEquals("p3", jar.getBsn());
-		// assertEquals("1.2.0", jar.getVersion());
-		//
-		// // -baseline specified
-		// builder.set("-baseline", "p3;version=1.1.0");
-		// jar = builder.getBaselineJar(false);
-		// assertEquals("p3", jar.getBsn());
-		// assertEquals("1.1.0", jar.getVersion());
-		//
-		// jar = builder.getBaselineJar(true);
-		// assertEquals("p3", jar.getBsn());
-		// assertEquals("1.1.0", jar.getVersion());
-
-	}
-
 	/**
 	 * When a JAR is build the manifest is not set in the resources but in a
 	 * instance var.
 	 *
 	 * @throws Exception
 	 */
+	@Test
 	public void testPrematureJar() throws Exception {
 		File file = IO.getFile(new File(""), "jar/osgi.jar");
 		try (Builder b1 = new Builder(); Builder b2 = new Builder();) {
@@ -312,7 +273,9 @@ public class BaselineTest extends TestCase {
 	 *
 	 * @throws Exception
 	 */
-	public void testRepository() throws Exception {
+	@Test
+	public void testRepository(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		Jar v1_2_0_a = mock(Jar.class);
 		when(v1_2_0_a.getVersion()).thenReturn("1.2.0.b");
 		when(v1_2_0_a.getBsn()).thenReturn("p3");
@@ -326,9 +289,9 @@ public class BaselineTest extends TestCase {
 		when(repo.getName()).thenReturn("Baseline");
 		when(repo.versions("p3")).thenReturn(new SortedList<>(new Version("1.1.0.a"), new Version("1.1.0.b"),
 			new Version("1.2.0.a"), new Version("1.2.0.b")));
-		getWorkspace().addBasicPlugin(repo);
+		getWorkspace(tmp).addBasicPlugin(repo);
 
-		Project p3 = getWorkspace().getProject("p3");
+		Project p3 = getWorkspace(tmp).getProject("p3");
 		p3.setBundleVersion("1.3.0");
 		ProjectBuilder builder = (ProjectBuilder) p3.getBuilder(null)
 			.getSubBuilder();
@@ -397,16 +360,16 @@ public class BaselineTest extends TestCase {
 	 *
 	 * @throws Exception
 	 */
-	public void testNothingInRepo() throws Exception {
-		File tmp = new File("tmp");
-		tmp.mkdirs();
+	@Test
+	public void testNothingInRepo(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		try {
 			RepositoryPlugin repo = mock(RepositoryPlugin.class);
 			when(repo.canWrite()).thenReturn(true);
 			when(repo.getName()).thenReturn("Baseline");
 			when(repo.versions("p3")).thenReturn(new TreeSet<>());
-			getWorkspace().addBasicPlugin(repo);
-			Project p3 = getWorkspace().getProject("p3");
+			getWorkspace(tmp).addBasicPlugin(repo);
+			Project p3 = getWorkspace(tmp).getProject("p3");
 			p3.setProperty(Constants.BASELINE, "*");
 			p3.setProperty(Constants.BASELINEREPO, "Baseline");
 			p3.setBundleVersion("0");
@@ -417,7 +380,23 @@ public class BaselineTest extends TestCase {
 			p3.build();
 			assertTrue(p3.check());
 
-			p3.setBundleVersion("5");
+			p3.setBundleVersion("1.0.1");
+			p3.build();
+			assertTrue(p3.check("There is no baseline for p3 in the baseline repo"));
+
+			p3.setBundleVersion("1.1");
+			p3.build();
+			assertTrue(p3.check("There is no baseline for p3 in the baseline repo"));
+
+			p3.setBundleVersion("2.0.0.XXXXXX");
+			p3.build();
+			assertTrue(p3.check());
+
+			p3.setBundleVersion("2.0.1");
+			p3.build();
+			assertTrue(p3.check("There is no baseline for p3 in the baseline repo"));
+
+			p3.setBundleVersion("2.1");
 			p3.build();
 			assertTrue(p3.check("There is no baseline for p3 in the baseline repo"));
 		} finally {
@@ -426,6 +405,7 @@ public class BaselineTest extends TestCase {
 	}
 
 	// Adding a method to a ProviderType produces a MINOR bump (1.0.0 -> 1.1.0)
+	@Test
 	public void testProviderTypeBump() throws Exception {
 		Processor processor = new Processor();
 
@@ -450,7 +430,8 @@ public class BaselineTest extends TestCase {
 	}
 
 	// Adding a method to a ConsumerType produces a MINOR bump (1.0.0 -> 2.0.0)
-	public static void testConsumerTypeBump() throws Exception {
+	@Test
+	public void testConsumerTypeBump() throws Exception {
 		Processor processor = new Processor();
 
 		DiffPluginImpl differ = new DiffPluginImpl();
@@ -472,6 +453,7 @@ public class BaselineTest extends TestCase {
 	}
 
 	// Adding a method to a ProviderType produces a MINOR bump (1.0.0 -> 1.1.0)
+	@Test
 	public void testBundleVersionBump() throws Exception {
 		Processor processor = new Processor();
 
@@ -491,6 +473,28 @@ public class BaselineTest extends TestCase {
 	}
 
 	// Adding a method to a ProviderType produces a MINOR bump (1.0.0 -> 1.1.0)
+	// in package, but bundle version should be ignored
+	@Test
+	public void testBundleVersionDiffignore() throws Exception {
+		Processor processor = new Processor();
+
+		DiffPluginImpl differ = new DiffPluginImpl();
+		differ.setIgnore("Bundle-Version");
+		Baseline baseline = new Baseline(processor, differ);
+
+		try (Jar older = new Jar(IO.getFile("testresources/api-orig.jar"));
+			Jar newer = new Jar(IO.getFile("testresources/api-providerbump.jar"));) {
+
+			baseline.baseline(newer, older, null);
+
+			BundleInfo bundleInfo = baseline.getBundleInfo();
+
+			assertFalse(bundleInfo.mismatch);
+		}
+	}
+
+	// Adding a method to a ProviderType produces a MINOR bump (1.0.0 -> 1.1.0)
+	@Test
 	public void testBundleVersionBumpDifferentSymbolicNames() throws Exception {
 		Processor processor = new Processor();
 
@@ -515,6 +519,7 @@ public class BaselineTest extends TestCase {
 
 	// Adding a method to an exported class produces a MINOR bump (1.0.0 ->
 	// 1.1.0)
+	@Test
 	public void testMinorChange() throws Exception {
 		Processor processor = new Processor();
 
@@ -535,6 +540,7 @@ public class BaselineTest extends TestCase {
 
 	// Adding a method to an exported class and unexporting a package produces a
 	// MINOR bump (1.0.0 -> 1.1.0)
+	@Test
 	public void testMinorAndRemovedChange() throws Exception {
 		Processor processor = new Processor();
 
@@ -555,6 +561,7 @@ public class BaselineTest extends TestCase {
 
 	// Deleting a protected field on a ProviderType API class produces a MINOR
 	// bump (1.0.0 -> 1.1.0)
+	@Test
 	public void testProviderProtectedFieldRemovedChange() throws Exception {
 		Processor processor = new Processor();
 
@@ -585,6 +592,7 @@ public class BaselineTest extends TestCase {
 
 	// Moving a package from the root into a jar on the Bundle-ClassPath
 	// should not result in DELETED
+	@Test
 	public void testMovePackageToBundleClassPath() throws Exception {
 		Processor processor = new Processor();
 
@@ -618,6 +626,7 @@ public class BaselineTest extends TestCase {
 	// baseline cannot find it. Since the class hierarchy was cut off, the
 	// baseline would _forget_ that every class inherits from Object, and _lose_
 	// Object's methods if not directly implemented.
+	@Test
 	public void testCutOffInheritance() throws Exception {
 		Processor processor = new Processor();
 
@@ -646,11 +655,11 @@ public class BaselineTest extends TestCase {
 
 			Diff packageDiff = change.packageDiff;
 
-			Collection<? extends Diff> children = packageDiff.getChildren();
+			Collection<Diff> children = packageDiff.getChildren();
 
 			assertEquals(5, children.size());
 
-			Iterator<? extends Diff> iterator = children.iterator();
+			Iterator<Diff> iterator = children.iterator();
 
 			Diff diff = iterator.next();
 			assertEquals(Delta.MICRO, diff.getDelta());
@@ -669,6 +678,7 @@ public class BaselineTest extends TestCase {
 	// <T extends Foo> T getFoo();
 	// or:
 	// <T extends Foo & Comparable<Foo>> T getFoo();
+	@Test
 	public void testExpandErasureOfMethodReturn() throws Exception {
 		Processor processor = new Processor();
 
@@ -697,11 +707,11 @@ public class BaselineTest extends TestCase {
 
 			Diff packageDiff = change.packageDiff;
 
-			Collection<? extends Diff> children = packageDiff.getChildren();
+			Collection<Diff> children = packageDiff.getChildren();
 
-			assertEquals(3, children.size());
+			assertEquals(4, children.size());
 
-			Iterator<? extends Diff> iterator = children.iterator();
+			Iterator<Diff> iterator = children.iterator();
 
 			Diff diff = iterator.next();
 			assertEquals(Delta.UNCHANGED, diff.getDelta());
