@@ -1,19 +1,25 @@
 package test.jpms;
 
 import static org.assertj.core.api.Assertions.assertThat;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 
 import java.io.DataInputStream;
 import java.io.File;
 import java.util.Arrays;
 
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.DisplayName;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.CsvSource;
+import org.junit.jupiter.params.provider.ValueSource;
 
 import aQute.bnd.classfile.ClassFile;
 import aQute.bnd.classfile.ModuleAttribute;
 import aQute.bnd.classfile.ModuleMainClassAttribute;
+import aQute.bnd.classfile.ModulePackagesAttribute;
 import aQute.bnd.osgi.Builder;
 import aQute.bnd.osgi.Constants;
 import aQute.bnd.osgi.Jar;
@@ -22,6 +28,94 @@ import aQute.lib.io.IO;
 
 public class JPMSModuleInfoPluginTest {
 
+	@ParameterizedTest(name = "Validate Module Access inputs with invalid content {arguments}")
+	@ValueSource(strings = {
+		"OPEN,SYNTHETIC,3", //
+		"OPEN,47,MANDATED", //
+		"99,MANDATED,SYNTHETIC", //
+		"foo,MANDATED,SYNTHETIC" //
+	})
+	@DisplayName("Validate Module Access inputs produce correct access")
+	public void moduleAccessWithException(String access) throws Exception {
+		Assertions.assertThrows(org.opentest4j.AssertionFailedError.class, () -> {
+			moduleAccess(access, 0);
+		});
+	}
+
+	@ParameterizedTest(name = "Validate Module Access inputs {arguments}")
+	@CsvSource(delimiterString = "|", value = {
+		"OPEN|32", //
+		"open|32", //
+		"0x0020|32", //
+		"0x20|32", //
+		"32|32", //
+		"SYNTHETIC|4096", //
+		"synthetic|4096", //
+		"0x1000|4096", //
+		"4096|4096", //
+		"MANDATED|32768", //
+		"mandated|32768", //
+		"0x8000|32768", //
+		"32768|32768", //
+		"OPEN,SYNTHETIC|4128", //
+		"OPEN,MANDATED|32800", //
+		"MANDATED,SYNTHETIC|36864" //
+	})
+	@DisplayName("Validate Module Access inputs produce correct access")
+	public void moduleAccess(String access, int flags) throws Exception {
+		try (Builder b = new Builder()) {
+			b.setProperty(Constants.JPMS_MODULE_INFO, "foo;access='" + access + "'");
+			b.setProperty(Constants.BUNDLE_SYMBOLICNAME, "foo");
+			b.setProperty(Constants.BUNDLE_VERSION, "1.2.7");
+			b.setProperty(Constants.PRIVATEPACKAGE, "test.jpms.a.*");
+			b.addClasspath(new File("bin_test"));
+			Jar jar = b.build();
+
+			if (!b.check())
+				fail();
+
+			Resource moduleInfo = jar.getResource(Constants.MODULE_INFO_CLASS);
+			assertNotNull(moduleInfo);
+
+			ClassFile module_info = ClassFile.parseClassFile(new DataInputStream(moduleInfo.openInputStream()));
+
+			assertThat(module_info.this_class).isEqualTo("module-info");
+
+			ModuleAttribute moduleAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModuleAttribute.class::isInstance)
+				.map(ModuleAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(moduleAttribute.module_name).isEqualTo("foo");
+			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
+			assertThat(moduleAttribute.module_flags).isEqualTo(flags);
+			assertThat(moduleAttribute.opens).hasSize(0);
+			assertThat(moduleAttribute.exports).hasSize(0);
+			assertThat(moduleAttribute.provides).hasSize(0);
+			assertThat(moduleAttribute.requires).hasSize(1)
+				.anyMatch(e -> e.requires.equals("java.base"));
+			assertThat(moduleAttribute.uses).hasSize(0);
+
+			ModuleMainClassAttribute moduleMainClassAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModuleMainClassAttribute.class::isInstance)
+				.map(ModuleMainClassAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/a");
+		}
+	}
+
 	@Test
 	public void moduleWithOptionsIgnore() throws Exception {
 		try (Builder b = new Builder()) {
@@ -55,7 +149,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/j"));
@@ -65,6 +159,15 @@ public class JPMSModuleInfoPluginTest {
 				.anyMatch(e -> e.requires.equals("java.base"))
 				.anyMatch(e -> e.requires.equals("geronimo-jcdi_2.0_spec"))
 				.anyMatch(e -> e.requires.equals("java.json"));
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/j");
 		}
 	}
 
@@ -101,7 +204,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/j"));
@@ -116,6 +219,15 @@ public class JPMSModuleInfoPluginTest {
 			assertThat(moduleAttribute.requires).filteredOn(e -> e.requires.equals("java.json.bind"))
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(ModuleAttribute.Require.ACC_STATIC_PHASE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/j");
 		}
 	}
 
@@ -153,7 +265,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/j"));
@@ -169,6 +281,15 @@ public class JPMSModuleInfoPluginTest {
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(
 					ModuleAttribute.Require.ACC_TRANSITIVE | ModuleAttribute.Require.ACC_STATIC_PHASE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/j");
 		}
 	}
 
@@ -176,8 +297,7 @@ public class JPMSModuleInfoPluginTest {
 	public void moduleWithOptionsMapModuleName() throws Exception {
 		try (Builder b = new Builder()) {
 			b.setProperty(Constants.JPMS_MODULE_INFO, "foo.module;version='1.2.7-module'");
-			b.setProperty(Constants.JPMS_MODULE_INFO_OPTIONS,
-				"java.enterprise;substitute=geronimo-jcdi_2.0_spec");
+			b.setProperty(Constants.JPMS_MODULE_INFO_OPTIONS, "java.enterprise;substitute=geronimo-jcdi_2.0_spec");
 			b.setProperty(Constants.BUNDLE_SYMBOLICNAME, "foo");
 			b.setProperty(Constants.BUNDLE_VERSION, "1.2.7");
 			b.setProperty(Constants.PRIVATEPACKAGE, "test.jpms.j.*");
@@ -206,7 +326,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/j"));
@@ -221,6 +341,15 @@ public class JPMSModuleInfoPluginTest {
 			assertThat(moduleAttribute.requires).filteredOn(e -> e.requires.equals("java.enterprise"))
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(ModuleAttribute.Require.ACC_TRANSITIVE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/j");
 		}
 	}
 
@@ -258,7 +387,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/j"));
@@ -277,6 +406,15 @@ public class JPMSModuleInfoPluginTest {
 			assertThat(moduleAttribute.requires).filteredOn(e -> e.requires.equals("geronimo-jcdi_2.0_spec"))
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(ModuleAttribute.Require.ACC_TRANSITIVE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/j");
 		}
 	}
 
@@ -313,7 +451,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/i"));
@@ -332,6 +470,15 @@ public class JPMSModuleInfoPluginTest {
 			assertThat(moduleAttribute.requires).filteredOn(e -> e.requires.equals("geronimo-jcdi_2.0_spec"))
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(ModuleAttribute.Require.ACC_TRANSITIVE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/i");
 		}
 	}
 
@@ -369,7 +516,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/i"));
@@ -388,6 +535,15 @@ public class JPMSModuleInfoPluginTest {
 			assertThat(moduleAttribute.requires).filteredOn(e -> e.requires.equals("geronimo-jcdi_2.0_spec"))
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(ModuleAttribute.Require.ACC_TRANSITIVE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/i");
 		}
 	}
 
@@ -424,7 +580,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/i"));
@@ -443,6 +599,15 @@ public class JPMSModuleInfoPluginTest {
 			assertThat(moduleAttribute.requires).filteredOn(e -> e.requires.equals("geronimo-jcdi_2.0_spec"))
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(ModuleAttribute.Require.ACC_TRANSITIVE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/i");
 		}
 	}
 
@@ -478,7 +643,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/i"));
@@ -498,6 +663,15 @@ public class JPMSModuleInfoPluginTest {
 			assertThat(moduleAttribute.requires).filteredOn(e -> e.requires.equals("geronimo-jcdi_2.0_spec"))
 				.flatExtracting(e -> Arrays.asList(e.requires_flags))
 				.containsExactlyInAnyOrder(ModuleAttribute.Require.ACC_TRANSITIVE);
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/i");
 		}
 	}
 
@@ -533,7 +707,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo.module");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7-module");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/i"));
@@ -567,6 +741,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/i");
 		}
 	}
 
@@ -602,7 +785,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/i"));
@@ -635,6 +818,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/i");
 		}
 	}
 
@@ -669,7 +861,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/i"));
@@ -689,6 +881,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/i");
 		}
 	}
 
@@ -722,7 +923,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(1)
 				.anyMatch(e -> e.exports.equals("test/jpms/h"));
@@ -745,6 +946,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/h");
 		}
 	}
 
@@ -795,6 +1005,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/g");
 		}
 	}
 
@@ -843,6 +1062,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/f");
 		}
 	}
 
@@ -874,7 +1102,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(2)
 				.anyMatch(e -> e.opens.equals("test/jpms/e/other"))
 				.anyMatch(e -> e.opens.equals("test/jpms/e"));
@@ -902,6 +1130,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/e", "test/jpms/e/other");
 		}
 	}
 
@@ -948,6 +1185,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/d");
 		}
 	}
 
@@ -979,7 +1225,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(0);
 			assertThat(moduleAttribute.provides).hasSize(0);
@@ -995,6 +1241,15 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleMainClassAttribute).isNotNull();
 			assertThat(moduleMainClassAttribute.main_class).isEqualTo("test/jpms/c/Foo");
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/c");
 		}
 	}
 
@@ -1026,7 +1281,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(2)
 				.anyMatch(e -> e.exports.equals("test/jpms/b/other"))
@@ -1054,6 +1309,16 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/b", "test/jpms/b/internal",
+				"test/jpms/b/other");
 		}
 	}
 
@@ -1085,7 +1350,7 @@ public class JPMSModuleInfoPluginTest {
 
 			assertThat(moduleAttribute.module_name).isEqualTo("foo");
 			assertThat(moduleAttribute.module_version).isEqualTo("1.2.7");
-			assertThat(moduleAttribute.module_flags).isEqualTo(0);
+			assertThat(moduleAttribute.module_flags).isEqualTo(32);
 			assertThat(moduleAttribute.opens).hasSize(0);
 			assertThat(moduleAttribute.exports).hasSize(0);
 			assertThat(moduleAttribute.provides).hasSize(0);
@@ -1100,6 +1365,15 @@ public class JPMSModuleInfoPluginTest {
 				.orElse(null);
 
 			assertThat(moduleMainClassAttribute).isNull();
+
+			ModulePackagesAttribute modulePackagesAttribute = Arrays.stream(module_info.attributes)
+				.filter(ModulePackagesAttribute.class::isInstance)
+				.map(ModulePackagesAttribute.class::cast)
+				.findFirst()
+				.orElse(null);
+
+			assertThat(modulePackagesAttribute).isNotNull();
+			assertThat(modulePackagesAttribute.packages).containsExactly("test/jpms/a");
 		}
 	}
 
