@@ -1,5 +1,6 @@
 package aQute.bnd.osgi;
 
+import static aQute.bnd.exceptions.FunctionWithException.asFunction;
 import static java.util.stream.Collectors.toList;
 
 import java.io.File;
@@ -16,10 +17,14 @@ import java.util.Collections;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Objects;
 import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.function.Function;
+import java.util.jar.JarFile;
 import java.util.jar.Manifest;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -29,7 +34,6 @@ import java.util.zip.ZipException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import aQute.bnd.apiguardian.APIGuardianAnnotations;
 import aQute.bnd.cdi.CDIAnnotations;
 import aQute.bnd.component.DSAnnotations;
 import aQute.bnd.differ.DiffPluginImpl;
@@ -57,11 +61,13 @@ import aQute.bnd.service.diff.Type;
 import aQute.bnd.service.specifications.BuilderSpecification;
 import aQute.bnd.stream.MapStream;
 import aQute.bnd.version.Version;
+import aQute.lib.collections.Logic;
 import aQute.lib.collections.MultiMap;
 import aQute.lib.hex.Hex;
 import aQute.lib.io.IO;
 import aQute.lib.regex.PatternConstants;
 import aQute.lib.strings.Strings;
+import aQute.lib.zip.ZipUtil;
 import aQute.libg.generics.Create;
 
 /**
@@ -70,25 +76,35 @@ import aQute.libg.generics.Create;
  * Import-Package: package-decl ( ',' package-decl )* @version $Revision: 1.27 $
  */
 public class Builder extends Analyzer {
-	private final static Logger		logger						= LoggerFactory.getLogger(Builder.class);
-	private final static Pattern	IR_PATTERN					= Pattern.compile("[{]?-?@?(?:[^=]+=)?\\s*([^}!]+).*");
-	private final DiffPluginImpl	differ						= new DiffPluginImpl();
-	private Pattern					xdoNotCopy					= null;
-	private static final int		SPLIT_MERGE_LAST			= 1;
-	private static final int		SPLIT_MERGE_FIRST			= 2;
-	private static final int		SPLIT_ERROR					= 3;
-	private static final int		SPLIT_FIRST					= 4;
-	private static final int		SPLIT_DEFAULT				= 0;
-	private final List<File>		sourcePath					= new ArrayList<>();
-	private final Make				make						= new Make(this);
-	private Instructions			defaultPreProcessMatcher	= null;
-	private BuilderInstructions		buildInstrs					= getInstructions(BuilderInstructions.class);
+	private final static Logger			logger						= LoggerFactory.getLogger(Builder.class);
+	private final static Pattern		IR_PATTERN					= Pattern
+		.compile("[{]?-?@?(?:[^=]+=)?\\s*([^}!]+).*");
+	private final DiffPluginImpl		differ						= new DiffPluginImpl();
+	private Pattern						xdoNotCopy					= null;
+	private static final int			SPLIT_MERGE_LAST			= 1;
+	private static final int			SPLIT_MERGE_FIRST			= 2;
+	private static final int			SPLIT_ERROR					= 3;
+	private static final int			SPLIT_FIRST					= 4;
+	private static final int			SPLIT_DEFAULT				= 0;
+	private final List<File>			sourcePath					= new ArrayList<>();
+	private final Make					make						= new Make(this);
+	private Instructions				defaultPreProcessMatcher	= null;
+	private BuilderInstructions			buildInstrs					= getInstructions(BuilderInstructions.class);
+	private final Map<Integer, String>	cachedSystemCalls;
 
 	public Builder(Processor parent) {
 		super(parent);
+		cachedSystemCalls = new ConcurrentHashMap<>();
 	}
 
-	public Builder() {}
+	public Builder(Builder parent) {
+		super(parent);
+		cachedSystemCalls = parent.cachedSystemCalls;
+	}
+
+	public Builder() {
+		cachedSystemCalls = new ConcurrentHashMap<>();
+	}
 
 	public Jar build() throws Exception {
 		logger.debug("build");
@@ -174,13 +190,14 @@ public class Builder extends Analyzer {
 		Jar dot = getJar();
 		if (dot == null) {
 			dot = new Jar("dot");
-
-			buildInstrs.compression()
-				.ifPresent(dot::setCompression);
-
-			dot.setReproducible(is(REPRODUCIBLE));
 			setJar(dot);
 		}
+
+		buildInstrs.compression()
+			.ifPresent(dot::setCompression);
+
+		dot.setReproducible(getProperty(REPRODUCIBLE));
+
 		try {
 			long modified = Long.parseLong(getProperty("base.modified"));
 			dot.updateModified(modified, "Base modified");
@@ -266,7 +283,7 @@ public class Builder extends Analyzer {
 		dot.getResources()
 			.keySet()
 			.stream()
-			.filter(path -> !pathStartsWith(path, "WEB-INF") && Arrays.stream(Constants.METAPACKAGES)
+			.filter(path -> !pathStartsWith(path, "WEB-INF") && Constants.METAPACKAGES.stream()
 				.noneMatch(meta -> pathStartsWith(path, meta)))
 			// we collect since we need to mutate the source set
 			.collect(toList())
@@ -386,7 +403,7 @@ public class Builder extends Analyzer {
 	@Override
 	protected Jar getExtra() throws Exception {
 		Parameters conditionals = getMergedParameters(CONDITIONAL_PACKAGE);
-		conditionals.putAll(getMergedParameters(CONDITIONALPACKAGE));
+		conditionals.putAll(decorated(CONDITIONALPACKAGE));
 		if (conditionals.isEmpty())
 			return null;
 		logger.debug("do Conditional Package {}", conditionals);
@@ -514,6 +531,10 @@ public class Builder extends Analyzer {
 		}
 	}
 
+	private static final String[] fixed = {
+		"packageinfo", "package.html", "module-info.java", "package-info.java"
+	};
+
 	/**
 	 * @throws IOException
 	 */
@@ -528,23 +549,17 @@ public class Builder extends Analyzer {
 			String sourcePath = typeRef.getSourcePath();
 			String packagePath = packageRef.getPath();
 
-			boolean found = false;
-			String[] fixed = {
-				"packageinfo", "package.html", "module-info.java", "package-info.java"
-			};
-
 			for (File root : getSourcePath()) {
 				File f = getFile(root, sourcePath);
 				if (f.exists()) {
-					found = true;
 					if (!packages.contains(packageRef)) {
 						packages.add(packageRef);
-						for (int j = 0; j < fixed.length; j++) {
+						for (String element : fixed) {
 							for (File sp : getSourcePath()) {
 								File bdir = getFile(sp, packagePath);
-								File ff = getFile(bdir, fixed[j]);
+								File ff = getFile(bdir, element);
 								if (ff.isFile()) {
-									String name = "OSGI-OPT/src/" + packagePath + "/" + fixed[j];
+									String name = "OSGI-OPT/src/" + packagePath + "/" + element;
 									dot.putResource(name, new FileResource(ff));
 									break;
 								}
@@ -571,9 +586,7 @@ public class Builder extends Analyzer {
 			String sp = mergeProperties(SOURCEPATH);
 			if (sp != null) {
 				Parameters map = parseHeader(sp);
-				for (Iterator<String> i = map.keySet()
-					.iterator(); i.hasNext();) {
-					String file = i.next();
+				for (String file : map.keySet()) {
 					if (!isDuplicate(file)) {
 						File f = getFile(file);
 						if (!f.isDirectory()) {
@@ -615,23 +628,40 @@ public class Builder extends Analyzer {
 				.forEachOrdered(path -> packages.add(path, srce));
 		}
 
-		Parameters includedPackages = getPrivatePackage();
+		Parameters private_package = getParameters(PRIVATE_PACKAGE);
+		Parameters privatepackage = decorated(PRIVATEPACKAGE);
+		Parameters testpackage = new Parameters();
+		Parameters includepackage = decorated(INCLUDEPACKAGE);
+
 		if (buildInstrs.undertest()) {
 			String h = mergeProperties(Constants.TESTPACKAGES, "test;presence:=optional");
-			includedPackages.putAll(parseHeader(h));
+			testpackage = parseHeader(h);
 		}
 
-		Parameters limboPackages = buildInstrs.includepackage();
-		includedPackages.putAll(limboPackages);
+		Parameters includedPackages = new Parameters();
+		includedPackages.putAll(private_package);
+		includedPackages.putAll(privatepackage);
+		includedPackages.putAll(testpackage);
+		includedPackages.putAll(includepackage);
 
 		if (!includedPackages.isEmpty()) {
 			Instructions privateFilter = new Instructions(includedPackages);
 			Set<Instruction> unused = doExpand(dot, packages, privateFilter);
 
+			unused.removeIf(Instruction::isAny); // allow 0 matches
+
 			if (!unused.isEmpty()) {
+
+				String header = Constants.PRIVATEPACKAGE;
+				if (Logic.hasOverlap(private_package.keySet(), unused))
+					header = Constants.PRIVATE_PACKAGE;
+				else if (Logic.hasOverlap(testpackage.keySet(), unused))
+					header = Constants.TESTPACKAGES;
+				else if (Logic.hasOverlap(includepackage.keySet(), unused))
+					header = Constants.INCLUDEPACKAGE;
+
 				warning(
-					"Unused " + Constants.PRIVATE_PACKAGE + " instructions, no such package(s) on the class path: %s",
-					unused).header(Constants.PRIVATE_PACKAGE)
+					"Unused %s instructions , no such package(s) on the class path: %s", header, unused).header(header)
 						.context(unused.iterator()
 							.next()
 							.getInput());
@@ -654,11 +684,6 @@ public class Builder extends Analyzer {
 	/**
 	 * Destructively filter the packages from the build up index. This index is
 	 * used by the Export Package as well as the Private Package
-	 *
-	 * @param jar
-	 * @param name
-	 * @param instructions
-	 * @throws Exception
 	 */
 	private Set<Instruction> doExpand(Jar jar, MultiMap<String, Jar> index, Instructions filter) throws Exception {
 		Set<Instruction> unused = Create.set();
@@ -729,9 +754,8 @@ public class Builder extends Analyzer {
 			return providers;
 
 		List<Jar> np = new ArrayList<>();
-		for (Iterator<Jar> i = providers.iterator(); i.hasNext();) {
-			Jar j = i.next();
-			if (from.matches(j.getName())) {
+		for (Jar j : providers) {
+			if (from.matches(j.getName()) ^ from.isNegated()) {
 				np.add(j);
 			}
 		}
@@ -740,11 +764,6 @@ public class Builder extends Analyzer {
 
 	/**
 	 * Copy the package from the providers based on the split package strategy.
-	 *
-	 * @param dest
-	 * @param providers
-	 * @param directory
-	 * @param splitStrategy
 	 */
 	private void copyPackage(Jar dest, List<Jar> providers, String path, int splitStrategy) {
 		switch (splitStrategy) {
@@ -779,12 +798,7 @@ public class Builder extends Analyzer {
 	}
 
 	/**
-	 * Cop
-	 *
-	 * @param dest
-	 * @param srce
-	 * @param path
-	 * @param overwriteResource
+	 * Copy
 	 */
 	private void copy(Jar dest, Jar srce, String path, boolean overwrite) {
 		logger.debug("copy d={} s={} p={}", dest, srce, path);
@@ -813,7 +827,6 @@ public class Builder extends Analyzer {
 
 	/**
 	 * Analyze the classpath for a split package
-	 *
 	 */
 	private String diagnostic(String pack, List<Jar> culprits) {
 		// Default is like merge-first, but with a warning
@@ -888,16 +901,15 @@ public class Builder extends Analyzer {
 	 * @throws FileNotFoundException
 	 */
 	private void doIncludeResources(Jar jar) throws Exception {
-		String includes = getProperty("Bundle-Includes");
-		if (includes == null) {
-			includes = mergeProperties(INCLUDERESOURCE);
-			if (includes == null || includes.length() == 0)
-				includes = mergeProperties(Constants.INCLUDE_RESOURCE);
-		} else
+		Parameters includes = parseHeader(getProperty("Bundle-Includes"));
+		if (includes.isEmpty()) {
+			includes = decorated(Constants.INCLUDERESOURCE);
+			includes.putAll(getMergedParameters(Constants.INCLUDE_RESOURCE));
+		} else {
 			warning("Please use -includeresource instead of Bundle-Includes");
+		}
 
 		doIncludeResource(jar, includes);
-
 	}
 
 	private void doIncludeResource(Jar jar, String includes) throws Exception {
@@ -937,15 +949,13 @@ public class Builder extends Analyzer {
 		}
 
 		if (source.startsWith("@")) {
-			extractFromJar(jar, source.substring(1), parts.length == 1 ? "" : destination, absentIsOk);
+			extractFromJar(jar, source.substring(1), parts.length == 1 ? "" : destination, absentIsOk, extra);
 		} else if (extra.containsKey("cmd")) {
 			doCommand(jar, source, destination, extra, preprocess, absentIsOk);
 		} else if (extra.containsKey(LITERAL_ATTRIBUTE)) {
 			String literal = extra.get(LITERAL_ATTRIBUTE);
 			Resource r = new EmbeddedResource(literal, 0L);
-			String x = extra.get("extra");
-			if (x != null)
-				r.setExtra(x);
+			addExtra(r, extra.get("extra"));
 			copy(jar, name, r, extra);
 			if (preprocess != null) {
 				warning("Preprocessing does not work for literals: %s", name);
@@ -1004,9 +1014,7 @@ public class Builder extends Analyzer {
 				}
 			} else {
 				Resource r = clazz.getResource();
-				String x = extra.get("extra");
-				if (x != null)
-					r.setExtra(x);
+				addExtra(r, extra.get("extra"));
 				copy(jar, name, r, extra);
 				if (preprocess != null) {
 					warning("Preprocessing does not work for class references: %s", name);
@@ -1015,6 +1023,15 @@ public class Builder extends Analyzer {
 		}
 	}
 
+	private void addExtra(Resource resource, String value) {
+		if (value == null) {
+			return;
+		}
+		String encoded = resource.getExtra();
+		byte[] extra = (encoded != null) ? Resource.decodeExtra(encoded) : null;
+		resource.setExtra(Resource.encodeExtra(ZipUtil.extraFieldFromString(extra, value)));
+	}
+
 	private Instructions getPreProcessMatcher(Map<String, String> extra) {
 		if (defaultPreProcessMatcher == null) {
 			String preprocessmatchers = mergeProperties(PREPROCESSMATCHERS);
@@ -1138,7 +1155,7 @@ public class Builder extends Analyzer {
 	private void traverse(List<String> paths, File item) {
 
 		if (item.isDirectory()) {
-			for (File sub : item.listFiles()) {
+			for (File sub : IO.listFiles(item)) {
 				traverse(paths, sub);
 			}
 		} else if (item.isFile())
@@ -1155,8 +1172,7 @@ public class Builder extends Analyzer {
 	 */
 	private long findLastModifiedWhileOlder(File file, long lastModified) {
 		if (file.isDirectory()) {
-			File children[] = file.listFiles();
-			for (File child : children) {
+			for (File child : IO.listFiles(file)) {
 				if (child.lastModified() > lastModified)
 					return child.lastModified();
 
@@ -1201,7 +1217,12 @@ public class Builder extends Analyzer {
 			return;
 		}
 
-		File[] fs = dir.listFiles(filter);
+		List<File> fs;
+		try (Stream<String> names = IO.listStream(dir)) {
+			fs = names.map(name -> new File(dir, name))
+				.filter(filter::accept)
+				.collect(toList());
+		}
 		for (File file : fs) {
 			if (file.isDirectory()) {
 				if (recursive) {
@@ -1221,7 +1242,7 @@ public class Builder extends Analyzer {
 				files.put(p, file);
 			}
 		}
-		if (fs.length == 0) {
+		if (fs.isEmpty()) {
 			File empty = new File(dir, Constants.EMPTY_HEADER);
 			files.put(appendPath(path, empty.getName()), empty);
 		}
@@ -1250,9 +1271,7 @@ public class Builder extends Analyzer {
 		} else {
 			Resource lastChance = make.process(source);
 			if (lastChance != null) {
-				String x = extra.get("extra");
-				if (x != null)
-					lastChance.setExtra(x);
+				addExtra(lastChance, extra.get("extra"));
 				copy(jar, destinationPath, lastChance, extra);
 			} else
 				error("Input file does not exist: %s", source).header(source)
@@ -1261,16 +1280,12 @@ public class Builder extends Analyzer {
 	}
 
 	/**
-	 * Extra resources from a Jar and add them to the given jar. The clause is
-	 * the
+	 * Extra resources from a Jar and add them to the given jar.
 	 *
-	 * @param jar
-	 * @param clauses
-	 * @param i
-	 * @throws ZipException
-	 * @throws IOException
+	 * @param extra
 	 */
-	private void extractFromJar(Jar jar, String source, String destination, boolean absentIsOk)
+	private void extractFromJar(Jar jar, String source, String destination, boolean absentIsOk,
+		Map<String, String> extra)
 		throws ZipException, IOException {
 		// Inline all resources and classes from another jar
 		// optionally appended with a modified regular expression
@@ -1289,11 +1304,24 @@ public class Builder extends Analyzer {
 
 			error("Can not find JAR file '%s'", source);
 		} else {
+			Function<String, String> nameMapper = Function.identity();
+			if (isTrue(extra.get("flatten:"))) {
+				nameMapper = path -> Strings.getLastSegment(path, '/');
+			} else if (instr != null) {
+				if (extra.containsKey("rename:")) {
+					Instruction in = instr;
+					String replacement = extra.get("rename:");
+					nameMapper = path -> in.getMatcher(path)
+							.replaceAll(replacement);
+				}
+			}
+
 			for (Jar j : sub)
-				addAll(jar, j, instr, destination);
+				addAll(jar, j, instr, destination, nameMapper);
 		}
 	}
 
+
 	/**
 	 * Add all the resources in the given jar that match the given filter.
 	 *
@@ -1311,17 +1339,22 @@ public class Builder extends Analyzer {
 	 * @param filter a pattern that should match the resoures in sub to be added
 	 */
 	public boolean addAll(Jar to, Jar sub, Instruction filter, String destination) {
+		return addAll(to, sub, filter, destination, Function.identity());
+	}
+
+	private boolean addAll(Jar to, Jar sub, Instruction filter, String destination, Function<String, String> modifier) {
 		boolean dupl = false;
 		for (String name : sub.getResources()
 			.keySet()) {
-			if ("META-INF/MANIFEST.MF".equals(name))
+			if (JarFile.MANIFEST_NAME.equals(name))
 				continue;
 
 			if (doNotCopy(Strings.getLastSegment(name, '/')))
 				continue;
 
 			if (filter == null || filter.matches(name) ^ filter.isNegated())
-				dupl |= to.putResource(Processor.appendPath(destination, name), sub.getResource(name), true);
+				dupl |= to.putResource(Processor.appendPath(destination, modifier.apply(name)), sub.getResource(name),
+					true);
 		}
 		return dupl;
 	}
@@ -1333,10 +1366,8 @@ public class Builder extends Analyzer {
 
 		logger.debug("copy d={} s={} path={}", jar, from, path);
 		if (from.isDirectory()) {
-
-			File files[] = from.listFiles();
-			for (int i = 0; i < files.length; i++) {
-				copy(jar, appendPath(path, files[i].getName()), files[i], preprocess, extra);
+			for (File file : IO.listFiles(from)) {
+				copy(jar, appendPath(path, file.getName()), file, preprocess, extra);
 			}
 		} else {
 			if (from.exists()) {
@@ -1344,9 +1375,7 @@ public class Builder extends Analyzer {
 				if (preprocess != null && preprocess.matches(path)) {
 					resource = new PreprocessResource(this, resource);
 				}
-				String x = extra.get("extra");
-				if (x != null)
-					resource.setExtra(x);
+				addExtra(resource, extra.get("extra"));
 				if (path.endsWith("/"))
 					path = path + from.getName();
 				copy(jar, path, resource, extra);
@@ -1367,8 +1396,8 @@ public class Builder extends Analyzer {
 	}
 
 	public void setSourcepath(File[] files) {
-		for (int i = 0; i < files.length; i++)
-			addSourcepath(files[i]);
+		for (File file : files)
+			addSourcepath(file);
 	}
 
 	public void addSourcepath(File cp) {
@@ -1395,6 +1424,7 @@ public class Builder extends Analyzer {
 			int n = 0;
 			for (String file : map.keySet()) {
 				Jar c = new Jar(getFile(file));
+				c.setDoNotTouchManifest();
 
 				buildInstrs.compression()
 					.ifPresent(c::setCompression);
@@ -1477,9 +1507,9 @@ public class Builder extends Analyzer {
 
 		Instructions instructions = new Instructions(subsMap);
 
-		List<File> members = new ArrayList<>(Arrays.asList(getBase().listFiles()));
+		List<File> members = IO.listFiles(getBase());
 
-		nextFile: while (members.size() > 0) {
+		nextFile: while (!members.isEmpty()) {
 
 			File file = members.remove(0);
 
@@ -1492,10 +1522,7 @@ public class Builder extends Analyzer {
 				p = p.getParent();
 			}
 
-			for (Iterator<Instruction> i = instructions.keySet()
-				.iterator(); i.hasNext();) {
-
-				Instruction instruction = i.next();
+			for (Instruction instruction : instructions.keySet()) {
 				if (instruction.matches(file.getName())) {
 
 					if (!instruction.isNegated()) {
@@ -1556,8 +1583,7 @@ public class Builder extends Analyzer {
 	 *
 	 */
 	public void removeBundleSpecificHeaders() {
-		Set<String> set = new HashSet<>(Arrays.asList(BUNDLE_SPECIFIC_HEADERS));
-		setForceLocal(set);
+		setForceLocal(BUNDLE_SPECIFIC_HEADERS);
 	}
 
 	/**
@@ -1701,21 +1727,19 @@ public class Builder extends Analyzer {
 	static JPMSAnnotations			moduleAnnotations		= new JPMSAnnotations();
 	static JPMSModuleInfoPlugin		moduleInfoPlugin		= new JPMSModuleInfoPlugin();
 	static SPIDescriptorGenerator	spiDescriptorGenerator	= new SPIDescriptorGenerator();
-	static APIGuardianAnnotations	apiGuardianAnnotations	= new APIGuardianAnnotations();
 
 	@Override
-	protected void setTypeSpecificPlugins(Set<Object> list) {
-		list.add(makeBnd);
-		list.add(makeCopy);
-		list.add(serviceComponent);
-		list.add(cdiAnnotations);
-		list.add(dsAnnotations);
-		list.add(metatypeAnnotations);
-		list.add(moduleAnnotations);
-		list.add(moduleInfoPlugin);
-		list.add(spiDescriptorGenerator);
-		list.add(apiGuardianAnnotations);
-		super.setTypeSpecificPlugins(list);
+	protected void setTypeSpecificPlugins(PluginsContainer pluginsContainer) {
+		pluginsContainer.add(makeBnd);
+		pluginsContainer.add(makeCopy);
+		pluginsContainer.add(serviceComponent);
+		pluginsContainer.add(cdiAnnotations);
+		pluginsContainer.add(dsAnnotations);
+		pluginsContainer.add(metatypeAnnotations);
+		pluginsContainer.add(moduleAnnotations);
+		pluginsContainer.add(moduleInfoPlugin);
+		pluginsContainer.add(spiDescriptorGenerator);
+		super.setTypeSpecificPlugins(pluginsContainer);
 	}
 
 	/**
@@ -1754,7 +1778,7 @@ public class Builder extends Analyzer {
 			Instructions instructions = new Instructions(entry.getValue()
 				.get("--pack"));
 
-			logger.debug("diff against {} --full={} --pack={} --warning={}", file, full, instructions);
+			logger.debug("diff against {} --full={} --pack={} --warning={}", file, full, instructions, warning);
 			for (Diff p : api.getChildren()) {
 				String pname = p.getName();
 				if (p.getType() == Type.PACKAGE && instructions.matches(pname)) {
@@ -1786,9 +1810,6 @@ public class Builder extends Analyzer {
 
 	/**
 	 * Show the diff recursively
-	 *
-	 * @param p
-	 * @param i
 	 */
 	private void show(Diff p, String indent, boolean warning) {
 		Delta d = p.getDelta();
@@ -1836,6 +1857,7 @@ public class Builder extends Analyzer {
 	 * resolve any symbolic reference.
 	 */
 	private final static Pattern	GITREF_P		= Pattern.compile("ref:\\s*(refs/(heads|tags|remotes)/(\\S+))\\s*");
+	private final static Pattern	GIT_WORKTREES_P	= Pattern.compile("gitdir:\\s*(\\S+)\\s*");
 
 	final static String				_githeadHelp	= "${githead}, provide the SHA for the current git head";
 
@@ -1846,55 +1868,80 @@ public class Builder extends Analyzer {
 		// Locate the .git directory
 		//
 
-		File rover = getBase();
-		while (rover != null && rover.isDirectory()) {
-			File headFile = IO.getFile(rover, ".git/HEAD");
-			if (headFile.isFile()) {
-				//
-				// The head is either a symref (ref:
-				// refs/(heads|tags|remotes)/<name>)
-				//
-				String head = IO.collect(headFile)
-					.trim();
-				if (!Hex.isHex(head)) {
-					//
-					// Should be a symref
-					//
-					Matcher m = GITREF_P.matcher(head);
-					if (m.matches()) {
-						String reference = m.group(1);
-						// so the commit is in the following path
-						File file = IO.getFile(rover, ".git/" + reference);
-						if (!file.isFile()) {
-							// sigh, gc'd. Is in .git/packed-refs
-							file = IO.getFile(rover, ".git/packed-refs");
-							if (file.isFile()) {
-								String refs = IO.collect(file);
-								Pattern packedReferenceLinePattern = Pattern
-									.compile("(" + PatternConstants.SHA1 + ")\\s+" + reference + "\\s*\n");
-								Matcher packedReferenceMatcher = packedReferenceLinePattern.matcher(refs);
-								if (packedReferenceMatcher.find()) {
-									head = packedReferenceMatcher.group(1);
-								} else
-									return ""; // give up
-							} else
-								return ""; // give up
+		File GIT_DIR = null;
+		for (File rover = getBase(); (rover != null) && rover.isDirectory(); rover = rover.getParentFile()) {
+			GIT_DIR = IO.getFile(rover, ".git");
+			if (GIT_DIR.exists()) {
+				while (GIT_DIR.isFile()) {
+					String content = IO.collect(GIT_DIR)
+						.trim();
+					Matcher m = GIT_WORKTREES_P.matcher(content);
+					if (!m.matches()) {
+						break;
+					}
+					GIT_DIR = IO.getFile(rover, m.group(1));
+				}
+				break;
+			}
+		}
+		if ((GIT_DIR == null) || !GIT_DIR.isDirectory()) {
+			// Cannot find git directory
+			return "";
+		}
+		File GIT_COMMON_DIR = GIT_DIR;
+		File commondir = IO.getFile(GIT_DIR, "commondir");
+		if (commondir.isFile()) {
+			String content = IO.collect(commondir)
+				.trim();
+			GIT_COMMON_DIR = IO.getFile(GIT_DIR, content);
+		}
+		File HEAD = IO.getFile(GIT_DIR, "HEAD");
+		if (!HEAD.isFile()) {
+			// Cannot find HEAD file
+			return "";
+		}
+		//
+		// The head is either a SHA1 or symref (ref:
+		// refs/(heads|tags|remotes)/<name>)
+		//
+		String content = IO.collect(HEAD)
+			.trim();
+		if (!Hex.isHex(content)) {
+			//
+			// Should be a symref
+			//
+			Matcher m = GITREF_P.matcher(content);
+			if (m.matches()) {
+				String symref = m.group(1);
+				// so the commit is in the following path
+				File file = IO.getFile(GIT_COMMON_DIR, symref);
+				if (!file.isFile()) {
+					// sigh, gc'd. Is in .git/packed-refs
+					file = IO.getFile(GIT_COMMON_DIR, "packed-refs");
+					if (file.isFile()) {
+						String refs = IO.collect(file);
+						Pattern packedReferenceLinePattern = Pattern
+							.compile("(" + PatternConstants.SHA1 + ")\\s+" + symref + "\\s*\n");
+						Matcher packedReferenceMatcher = packedReferenceLinePattern.matcher(refs);
+						if (packedReferenceMatcher.find()) {
+							content = packedReferenceMatcher.group(1);
 						} else {
-							head = IO.collect(file);
+							return ""; // give up
 						}
 					} else {
-						error(
-							"Git repo seems corrupt. It exists, find the HEAD but the content is neither hex nor a sym-ref: %s",
-							head);
+						return ""; // give up
 					}
+				} else {
+					content = IO.collect(file);
 				}
-				return head.trim()
-					.toUpperCase();
+			} else {
+				error(
+					"Git repo seems corrupt. It exists, find the HEAD but the content is neither hex nor a sym-ref: %s",
+					content);
 			}
-			rover = rover.getParentFile();
 		}
-		// Cannot find git directory
-		return "";
+		return content.trim()
+			.toUpperCase(Locale.ROOT);
 	}
 
 	/**
@@ -1977,11 +2024,24 @@ public class Builder extends Analyzer {
 			setProperty(Constants.REQUIRE_CAPABILITY, new Parameters(spec.requireCapability).toString());
 		}
 
-		MapStream.of(spec.other)
-			.forEach(this::setProperty);
+		spec.other.forEach(this::setProperty);
 
 		return this;
 
 	}
 
+	/**
+	 * We override system so that, for the duration of a build operation, we may
+	 * cache the result of a given system call. In a large build with many
+	 * 'make' build jars, such as some OSGi CT build projects, we may call
+	 * system hundreds of time to compute headers based upon git information
+	 * such as 'git describe'. Since this information will not change during the
+	 * course of a single build operation, we cache results to only call once.
+	 */
+	@Override
+	public String system(boolean allowFail, String command, String input) throws IOException, InterruptedException {
+		Integer key = Objects.hash(command, input);
+		return cachedSystemCalls.computeIfAbsent(key, asFunction(k -> super.system(allowFail, command, input)));
+	}
+
 }
