@@ -27,6 +27,18 @@ import java.util.Objects;
 import java.util.Optional;
 import java.util.stream.StreamSupport;
 
+import aQute.bnd.build.Container;
+import aQute.bnd.build.Container.TYPE;
+import aQute.bnd.build.Workspace;
+import aQute.bnd.exceptions.Exceptions;
+import aQute.bnd.exporter.executable.ExecutableJarExporter;
+import aQute.bnd.exporter.runbundles.RunbundlesExporter;
+import aQute.bnd.osgi.About;
+import aQute.bnd.osgi.Constants;
+import aQute.bnd.stream.MapStream;
+import aQute.bnd.unmodifiable.Maps;
+import aQute.lib.io.IO;
+import aQute.lib.strings.Strings;
 import org.gradle.api.Action;
 import org.gradle.api.GradleException;
 import org.gradle.api.JavaVersion;
@@ -60,6 +72,7 @@ import org.gradle.api.tasks.Delete;
 import org.gradle.api.tasks.SourceSet;
 import org.gradle.api.tasks.SourceSetContainer;
 import org.gradle.api.tasks.TaskContainer;
+import org.gradle.api.tasks.TaskInputFilePropertyBuilder;
 import org.gradle.api.tasks.TaskProvider;
 import org.gradle.api.tasks.bundling.AbstractArchiveTask;
 import org.gradle.api.tasks.compile.AbstractCompile;
@@ -70,19 +83,6 @@ import org.gradle.language.base.plugins.LifecycleBasePlugin;
 import org.gradle.process.CommandLineArgumentProvider;
 import org.slf4j.LoggerFactory;
 
-import aQute.bnd.build.Container;
-import aQute.bnd.build.Container.TYPE;
-import aQute.bnd.build.Workspace;
-import aQute.bnd.exceptions.Exceptions;
-import aQute.bnd.exporter.executable.ExecutableJarExporter;
-import aQute.bnd.exporter.runbundles.RunbundlesExporter;
-import aQute.bnd.osgi.About;
-import aQute.bnd.osgi.Constants;
-import aQute.bnd.stream.MapStream;
-import aQute.bnd.unmodifiable.Maps;
-import aQute.lib.io.IO;
-import aQute.lib.strings.Strings;
-
 /**
  * BndPlugin for Gradle.
  * <p>
@@ -104,6 +104,11 @@ public class BndPlugin implements Plugin<Project> {
 	private ObjectFactory					objects;
 	private aQute.bnd.build.Project			bndProject;
 
+	/**
+	 * Default public constructor.
+	 */
+	public BndPlugin() {}
+
 	/**
 	 * Apply the {@code biz.aQute.bnd} plugin to the specified project.
 	 */
@@ -210,10 +215,14 @@ public class BndPlugin implements Plugin<Project> {
 					 * Workspace and project configuration changes should
 					 * trigger task
 					 */
-					t.getInputs()
+					TaskInputFilePropertyBuilder bndConfigurationInput = t.getInputs()
 						.files(bndConfiguration())
 						.withPathSensitivity(RELATIVE)
 						.withPropertyName("bndConfiguration");
+					if (isGradleCompatible("7.2")) {
+						bndConfigurationInput.normalizeLineEndings();
+					}
+
 					t.getOutputs()
 						.dirs(bndProject.getGenerate()
 							.getOutputDirs())
@@ -560,10 +569,13 @@ public class BndPlugin implements Plugin<Project> {
 					 * Workspace and project configuration changes should
 					 * trigger jar task
 					 */
-					t.getInputs()
+					TaskInputFilePropertyBuilder bndConfigurationInput = t.getInputs()
 						.files(bndConfiguration())
 						.withPathSensitivity(RELATIVE)
 						.withPropertyName("bndConfiguration");
+					if (isGradleCompatible("7.2")) {
+						bndConfigurationInput.normalizeLineEndings();
+					}
 					t.getOutputs()
 						.files(deliverables)
 						.withPropertyName("artifacts");
@@ -667,7 +679,6 @@ public class BndPlugin implements Plugin<Project> {
 			TaskProvider<TestOSGi> testOSGi = tasks.register("testOSGi", TestOSGi.class, t -> {
 				t.setDescription(
 					"Runs the OSGi JUnit tests by launching a framework and running the tests in the launched framework.");
-				t.setGroup(LifecycleBasePlugin.VERIFICATION_GROUP);
 				t.setEnabled(
 					!bndProject.is(Constants.NOJUNITOSGI) && !bndProject.getUnprocessedProperty(Constants.TESTCASES, "")
 						.isEmpty());
@@ -732,7 +743,6 @@ public class BndPlugin implements Plugin<Project> {
 			List<TaskProvider<Export>> exportTasks = MapStream.of(bndruns)
 				.mapToObj((name, runFile) -> tasks.register("export.".concat(name), Export.class, t -> {
 					t.setDescription(String.format("Export the %s file.", runFile.getName()));
-					t.setGroup(PublishingPlugin.PUBLISH_TASK_GROUP);
 					t.dependsOn(assemble);
 					t.getBndrun()
 						.fileValue(runFile);
@@ -751,7 +761,6 @@ public class BndPlugin implements Plugin<Project> {
 				.mapToObj((name, runFile) -> tasks.register("runbundles.".concat(name), Export.class, t -> {
 					t.setDescription(
 						String.format("Create a distribution of the runbundles in the %s file.", runFile.getName()));
-					t.setGroup(PublishingPlugin.PUBLISH_TASK_GROUP);
 					t.dependsOn(assemble);
 					t.getBndrun()
 						.fileValue(runFile);
@@ -769,7 +778,6 @@ public class BndPlugin implements Plugin<Project> {
 			List<TaskProvider<Resolve>> resolveTasks = MapStream.of(bndruns)
 				.mapToObj((name, runFile) -> tasks.register("resolve.".concat(name), Resolve.class, t -> {
 					t.setDescription(String.format("Resolve the runbundles required for %s file.", runFile.getName()));
-					t.setGroup(PublishingPlugin.PUBLISH_TASK_GROUP);
 					t.dependsOn(assemble);
 					t.getBndrun()
 						.fileValue(runFile);
@@ -778,14 +786,13 @@ public class BndPlugin implements Plugin<Project> {
 
 			TaskProvider<Task> resolve = tasks.register("resolve", t -> {
 				t.setDescription("Resolve the runbundles required for each of the bndrun files.");
-				t.setGroup(PublishingPlugin.PUBLISH_TASK_GROUP);
+				t.setGroup(LifecycleBasePlugin.VERIFICATION_GROUP);
 				t.dependsOn(resolveTasks);
 			});
 
 			bndruns.forEach((name, runFile) -> {
 				tasks.register("run.".concat(name), Bndrun.class, t -> {
 					t.setDescription(String.format("Run the bndrun file %s.", runFile.getName()));
-					t.setGroup(PublishingPlugin.PUBLISH_TASK_GROUP);
 					t.dependsOn(assemble);
 					t.getBndrun()
 						.fileValue(runFile);
@@ -796,7 +803,6 @@ public class BndPlugin implements Plugin<Project> {
 				tasks.register("testrun.".concat(name), TestOSGi.class, t -> {
 					t.setDescription(
 						String.format("Runs the OSGi JUnit tests in the bndrun file %s.", runFile.getName()));
-					t.setGroup(LifecycleBasePlugin.VERIFICATION_GROUP);
 					t.dependsOn(assemble);
 					t.getBndrun()
 						.fileValue(runFile);
