@@ -23,29 +23,44 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.nio.file.Files;
 import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Objects;
 import java.util.Optional;
 import java.util.Properties;
 import java.util.Set;
 import java.util.jar.Manifest;
-import java.util.stream.Collectors;
 import java.util.stream.Stream;
 import java.util.zip.ZipException;
 import java.util.zip.ZipFile;
 
+import aQute.bnd.build.Project;
+import aQute.bnd.exceptions.Exceptions;
+import aQute.bnd.header.OSGiHeader;
+import aQute.bnd.maven.PomPropertiesResource;
+import aQute.bnd.maven.lib.configuration.BeanProperties;
+import aQute.bnd.maven.lib.configuration.BndConfiguration;
+import aQute.bnd.osgi.Builder;
+import aQute.bnd.osgi.Constants;
+import aQute.bnd.osgi.FileResource;
+import aQute.bnd.osgi.Jar;
+import aQute.bnd.osgi.Processor;
+import aQute.bnd.osgi.Resource;
+import aQute.bnd.version.MavenVersion;
+import aQute.bnd.version.Version;
+import aQute.lib.io.IO;
+import aQute.lib.strings.Strings;
+import aQute.service.reporter.Report.Location;
 import org.apache.maven.artifact.Artifact;
+import org.apache.maven.artifact.handler.ArtifactHandler;
+import org.apache.maven.artifact.handler.manager.ArtifactHandlerManager;
 import org.apache.maven.artifact.resolver.filter.ScopeArtifactFilter;
 import org.apache.maven.model.Developer;
 import org.apache.maven.model.License;
-import org.apache.maven.model.Plugin;
-import org.apache.maven.model.PluginExecution;
-import org.apache.maven.model.PluginManagement;
 import org.apache.maven.plugin.AbstractMojo;
 import org.apache.maven.plugin.MojoExecution;
 import org.apache.maven.plugin.MojoExecutionException;
@@ -57,46 +72,40 @@ import org.apache.maven.project.MavenProjectHelper;
 import org.apache.maven.settings.Settings;
 import org.apache.maven.shared.mapping.MappingUtils;
 import org.codehaus.plexus.util.StringUtils;
-import org.codehaus.plexus.util.xml.Xpp3Dom;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.sonatype.plexus.build.incremental.BuildContext;
 
-import aQute.bnd.build.Project;
-import aQute.bnd.exceptions.Exceptions;
-import aQute.bnd.header.OSGiHeader;
-import aQute.bnd.maven.PomPropertiesResource;
-import aQute.bnd.maven.lib.configuration.BeanProperties;
-import aQute.bnd.osgi.Builder;
-import aQute.bnd.osgi.Constants;
-import aQute.bnd.osgi.FileResource;
-import aQute.bnd.osgi.Jar;
-import aQute.bnd.osgi.Processor;
-import aQute.bnd.osgi.Resource;
-import aQute.bnd.version.MavenVersion;
-import aQute.bnd.version.Version;
-import aQute.lib.io.IO;
-import aQute.lib.strings.Strings;
-import aQute.lib.utf8properties.UTF8Properties;
-import aQute.service.reporter.Report.Location;
-
+/**
+ * Abstract base class for all bnd-maven-plugin mojos.
+ */
 public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 	protected final Logger	logger					= LoggerFactory.getLogger(getClass());
-	static final String		MANIFEST_LAST_MODIFIED	= "aQute.bnd.maven.plugin.BndMavenPlugin.manifestLastModified";
+	static final String     LAST_MODIFIED           = "aQute.bnd.maven.plugin.BndMavenPlugin.lastModified";
 	static final String		MARKED_FILES			= "aQute.bnd.maven.plugin.BndMavenPlugin.markedFiles";
 	static final String		PACKAGING_JAR			= "jar";
 	static final String		PACKAGING_WAR			= "war";
 	static final String		TSTAMP					= "${tstamp}";
 	static final String		SNAPSHOT				= "SNAPSHOT";
 
-	@Parameter(defaultValue = "${project.build.directory}", readonly = true)
-	File					targetDir;
-
+	/**
+	 * Whether to include the contents of the {@code classesDir} directory
+	 * in the generated bundle.
+	 */
 	@Parameter(defaultValue = "true")
 	boolean					includeClassesDir;
 
-	@Parameter(defaultValue = "${project.build.directory}/${project.build.finalName}")
-	File					warOutputDir;
+	/**
+	 * The directory where the webapp is built when packaging is {@code war}.
+	 */
+	@Parameter(alias = "warOutputDir", defaultValue = "${project.build.directory}/${project.build.finalName}")
+	File					webappDirectory;
+
+	/**
+	 * The name of the created artifact (excluding extension and classifier).
+	 */
+	@Parameter(defaultValue = "${project.build.finalName}", readonly = true)
+	String					finalName;
 
 	@Parameter(defaultValue = "${project}", required = true, readonly = true)
 	MavenProject			project;
@@ -107,14 +116,28 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 	@Parameter(defaultValue = "${mojoExecution}", readonly = true)
 	MojoExecution			mojoExecution;
 
+	/**
+	 * The list of maven packaging types for which the plugin will execute.
+	 */
 	@Parameter(property = "bnd.packagingTypes", defaultValue = PACKAGING_JAR + "," + PACKAGING_WAR)
 	List<String>			packagingTypes;
 
+	/**
+	 * Skip processing if {@link #includeClassesDir} is {@code true} and the
+	 * {@code classesDir} directory is empty.
+	 */
 	@Parameter(property = "bnd.skipIfEmpty", defaultValue = "false")
 	boolean					skipIfEmpty;
 
+	/**
+	 * Timestamp for reproducible output archive entries, either formatted as ISO 8601
+	 * {@code yyyy-MM-dd'T'HH:mm:ssXXX} or as an int representing seconds since the epoch.
+	 *
+	 * @see <a href="https://maven.apache.org/guides/mini/guide-reproducible-builds.html">Configuring
+	 * for Reproducible Builds</a>
+	 */
 	@Parameter(defaultValue = "${project.build.outputTimestamp}")
-	private String			outputTimestamp;
+	String			outputTimestamp;
 
 	/**
 	 * File path to a bnd file containing bnd instructions for this project.
@@ -125,21 +148,23 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 	 * instructions, if any, for the parent project.
 	 */
 	@Parameter(defaultValue = Project.BNDFILE)
-	// This is not used and is for doc only; see loadProjectProperties
+	// This is not used and is for doc only; see {@link
+	// BndConfiguration#loadProperties(Processor)}
 	@SuppressWarnings("unused")
 	String					bndfile;
 
 	/**
 	 * Bnd instructions for this project specified directly in the pom file.
 	 * This is generally be done using a {@code <![CDATA[]]>} section. If the
-	 * projects has a {@link #bndfile bnd file}, then this configuration element
+	 * projects has a {@link #bndfile}, then this configuration element
 	 * is ignored.
 	 * <p>
 	 * The bnd instructions for this project are merged with the bnd
 	 * instructions, if any, for the parent project.
 	 */
 	@Parameter
-	// This is not used and is for doc only; see loadProjectProperties
+	// This is not used and is for doc only; See {@link
+	// BndConfiguration#loadProperties(Processor)}
 	@SuppressWarnings("unused")
 	String					bnd;
 
@@ -149,6 +174,9 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 	@Component
 	MavenProjectHelper		projectHelper;
 
+	@Component
+	ArtifactHandlerManager artifactHandlerManager;
+
 	File					propertiesFile;
 
 	public abstract File getSourceDir();
@@ -171,10 +199,14 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 		return Optional.empty();
 	}
 
+	File getWebappDirectory() {
+		return webappDirectory;
+	}
+
 	@Override
 	public void execute() throws MojoExecutionException, MojoFailureException {
-		// Exit without generating anything if this is neither a jar or war
-		// project. Probably it's just a parent project.
+		// Exit without generating anything if the project packaging is not a
+		// packaging type. Probably it's just a parent project.
 		if (!packagingTypes.contains(project.getPackaging())) {
 			logger.debug("skip project with packaging=" + project.getPackaging());
 			return;
@@ -207,8 +239,8 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 			builder.setTrace(logger.isDebugEnabled());
 
 			builder.setBase(project.getBasedir());
-			propertiesFile = loadProperties(builder);
-			builder.setProperty("project.output", targetDir.getCanonicalPath());
+			propertiesFile = new BndConfiguration(project, mojoExecution).loadProperties(builder);
+			builder.setProperty("project.output", getClassesDir().getCanonicalPath());
 
 			// If no bundle to be built, we have nothing to do
 			if (Processor.isTrue(builder.getProperty(Constants.NOBUNDLES))) {
@@ -225,12 +257,12 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 			// always add the outputDirectory to the classpath, but
 			// handle projects with no output directory, like
 			// 'test-wrapper-bundle'
-			if (getClassesDir().isDirectory()) {
-				builder.addClasspath(getClassesDir());
+			if (classesDir.isDirectory()) {
+				builder.addClasspath(classesDir);
 
 				Jar classesDirJar;
 				if (includeClassesDir) {
-					classesDirJar = new Jar(project.getName(), getClassesDir());
+					classesDirJar = new Jar(project.getName(), classesDir);
 				} else {
 					classesDirJar = new Jar(project.getName()); // empty jar
 				}
@@ -242,16 +274,13 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 			boolean hasWablibs = builder.getProperty(Constants.WABLIB) != null;
 			String wabProperty = builder.getProperty(Constants.WAB);
 
-			File outputDir = getOutputDir();
-
 			if (isWab) {
 				if (wabProperty == null) {
 					builder.setProperty(Constants.WAB, "");
 				}
-				outputDir = warOutputDir;
 				logger
 					.info("WAB mode enabled. Bnd output will be expanded into the 'maven-war-plugin' <webappDirectory>:"
-						+ outputDir);
+						+ getWebappDirectory());
 			} else if ((wabProperty != null) || hasWablibs) {
 				throw new MojoFailureException(
 					Constants.WAB + " & " + Constants.WABLIB + " are not supported with packaging 'jar'");
@@ -276,8 +305,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 					builder.updateModified(cpeJar.lastModified(), cpe.getPath());
 					buildpath.add(cpeJar);
 				} else {
-					if (!artifact.getType()
-						.equals("jar")) {
+					if (!cpe.getName().endsWith(".jar")) {
 						/*
 						 * Check if it is a valid zip file. We don't create a
 						 * Jar object here because we want to avoid the cost of
@@ -302,8 +330,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 			}
 
 			if (!wablibs.isEmpty()) {
-				String wablib = wablibs.stream()
-					.collect(Collectors.joining(","));
+				String wablib = String.join(",", wablibs);
 				builder.setProperty(Constants.WABLIB, wablib);
 			}
 
@@ -313,7 +340,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 			logger.debug("builder classpath: {}", builder.getProperty("project.buildpath"));
 
 			// Compute bnd sourcepath
-			boolean delta = !buildContext.isIncremental() || manifestOutOfDate();
+			boolean delta = !buildContext.isIncremental() || outOfDate();
 			List<File> sourcepath = new ArrayList<>();
 			if (getSourceDir().exists()) {
 				sourcepath.add(getSourceDir().getCanonicalFile());
@@ -496,30 +523,37 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 				// Build bnd Jar (in memory)
 				Jar bndJar = builder.build();
 
-				String goal = mojoExecution.getMojoDescriptor()
-					.getGoal();
-
 				// If a bnd-maven-plugin packaging goal is used it means
 				// this execution is responsible for creating and attaching the
 				// target artifact to the project
+				String goal = mojoExecution.getMojoDescriptor()
+					.getGoal();
 				if (isPackagingGoal(goal)) {
 					// However, if extensions for this plugin are not enabled
-					// the maven-(j|w)wa-plugin will also execute, resulting in
+					// the maven-(jar|war)-plugin will also execute, resulting in
 					// conflicting results
-					if (mojoExecution.getPlugin()
+					if (!mojoExecution.getPlugin()
 						.isExtensions()) {
-						// Add META-INF/maven metadata to jar
-						addMavenMetadataToJar(bndJar);
-						// Write the jar directly and attach it to the project
-						attachArtifactToProject(bndJar);
-					} else {
 						throw new MojoExecutionException(String.format(
 							"In order to use the bnd-maven-plugin packaging goal %s, <extensions>true</extensions> must be set on the plugin",
 							goal));
 					}
+					if (isWab) {
+						// Write Jar into webappDirectory
+						File outputDirectory = getWebappDirectory();
+						writeContent(bndJar, outputDirectory);
+						File manifestPath = new File(outputDirectory, bndJar.getManifestName());
+						writeManifest(bndJar, manifestPath);
+					}
+					// Add META-INF/maven metadata to jar
+					addMavenMetadataToJar(bndJar);
+					// Write the jar directly and attach it to the project
+					attachArtifactToProject(bndJar);
 				} else {
-					// Expand Jar into target/classes
-					expandJar(bndJar, outputDir);
+					File outputDirectory = isWab ? getWebappDirectory() : getOutputDir();
+					// Write Jar content into outputDirectory
+					writeContent(bndJar, outputDirectory);
+					writeManifest(bndJar, getManifestPath());
 				}
 			} else {
 				logger.debug("No build");
@@ -573,15 +607,22 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 	}
 
 	private void attachArtifactToProject(Jar bndJar) throws Exception {
-		File artifactFile = createArtifactFile();
-		File outputDir = artifactFile.getParentFile();
+		File artifactFile = getArtifactFile();
+		if (outOfDate(artifactFile) || artifactFile.lastModified() < bndJar.lastModified()) {
+			if (logger.isDebugEnabled()) {
+				if (artifactFile.exists())
+					logger.debug(String.format("Updating lastModified: %tF %<tT.%<tL '%s'", artifactFile.lastModified(),
+						artifactFile));
+				else
+					logger.debug("Creating '{}'", artifactFile);
+			}
 
-		if (!outputDir.exists()) {
-			IO.mkdirs(outputDir);
-		}
-
-		try (OutputStream os = buildContext.newFileOutputStream(artifactFile)) {
-			bndJar.write(os);
+			Files.createDirectories(artifactFile.toPath()
+				.getParent());
+			try (OutputStream os = buildContext.newFileOutputStream(artifactFile)) {
+				bndJar.write(os);
+			}
+			buildContext.setValue(LAST_MODIFIED, artifactFile.lastModified());
 		}
 
 		// If there is a classifier artifact must be attached to the
@@ -620,112 +661,26 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 		bndJar.putResource(pomProperties.getWhere(), pomProperties);
 	}
 
-	private File createArtifactFile() {
-		return new File(targetDir, project.getBuild()
-			.getFinalName()
-			+ getClassifier().map("-"::concat)
-				.orElse("")
-			+ "." + project.getPackaging());
+	private File getArtifactFile() {
+		return new File(getOutputDir(), finalName + getClassifier().map("-"::concat)
+			.orElse("") + "." + getExtension(project.getPackaging()));
+	}
+
+	private String getExtension(String type) {
+		ArtifactHandler artifactHandler = artifactHandlerManager.getArtifactHandler(type);
+		if (artifactHandler != null) {
+			return artifactHandler.getExtension();
+		}
+		return type;
 	}
 
 	private String createArtifactName(Artifact artifact) {
 		String classifier = artifact.getClassifier();
 		if ((classifier == null) || classifier.isEmpty()) {
-			return String.format("%s-%s.%s", artifact.getArtifactId(), artifact.getVersion(), artifact.getType());
+			return String.format("%s-%s.%s", artifact.getArtifactId(), artifact.getVersion(), getExtension(artifact.getType()));
 		}
 		return String.format("%s-%s-%s.%s", artifact.getArtifactId(), artifact.getVersion(), classifier,
-			artifact.getType());
-	}
-
-	private File loadProperties(Builder builder) throws Exception {
-		// Load parent project properties first
-		loadParentProjectProperties(builder, project);
-
-		// Load current project properties
-		Xpp3Dom configuration = Optional.ofNullable(project.getBuildPlugins())
-			.flatMap(this::getConfiguration)
-			.orElseGet(this::defaultConfiguration);
-		return loadProjectProperties(builder, project, project, configuration);
-	}
-
-	private void loadParentProjectProperties(Builder builder, MavenProject currentProject) throws Exception {
-		MavenProject parentProject = currentProject.getParent();
-		if (parentProject == null) {
-			return;
-		}
-		loadParentProjectProperties(builder, parentProject);
-
-		// Get configuration from parent project
-		Xpp3Dom configuration = Optional.ofNullable(parentProject.getBuildPlugins())
-			.flatMap(this::getConfiguration)
-			.orElse(null);
-		if (configuration != null) {
-			// Load parent project's properties
-			loadProjectProperties(builder, parentProject, parentProject, configuration);
-			return;
-		}
-
-		// Get configuration in project's pluginManagement
-		configuration = Optional.ofNullable(currentProject.getPluginManagement())
-			.map(PluginManagement::getPlugins)
-			.flatMap(this::getConfiguration)
-			.orElseGet(this::defaultConfiguration);
-		// Load properties from parent project's bnd file or configuration in
-		// project's pluginManagement
-		loadProjectProperties(builder, parentProject, currentProject, configuration);
-	}
-
-	private File loadProjectProperties(Builder builder, MavenProject bndProject, MavenProject pomProject,
-		Xpp3Dom configuration) throws Exception {
-		// check for bnd file configuration
-		File baseDir = bndProject.getBasedir();
-		if (baseDir != null) { // file system based pom
-			File pomFile = bndProject.getFile();
-			builder.updateModified(pomFile.lastModified(), "POM: " + pomFile);
-			// check for bnd file
-			Xpp3Dom bndfileElement = configuration.getChild("bndfile");
-			String bndFileName = (bndfileElement != null) ? bndfileElement.getValue() : Project.BNDFILE;
-			File bndFile = IO.getFile(baseDir, bndFileName);
-			if (bndFile.isFile()) {
-				logger.debug("loading bnd properties from file: {}", bndFile);
-				// we use setProperties to handle -include
-				builder.setProperties(bndFile.getParentFile(), builder.loadProperties(bndFile));
-				return bndFile;
-			}
-			// no bnd file found, so we fall through
-		}
-
-		// check for bnd-in-pom configuration
-		baseDir = pomProject.getBasedir();
-		File pomFile = pomProject.getFile();
-		if (baseDir != null) {
-			builder.updateModified(pomFile.lastModified(), "POM: " + pomFile);
-		}
-		Xpp3Dom bndElement = configuration.getChild("bnd");
-		if (bndElement != null) {
-			logger.debug("loading bnd properties from bnd element in pom: {}", pomProject);
-			UTF8Properties properties = new UTF8Properties();
-			properties.load(bndElement.getValue(), pomFile, builder);
-			// we use setProperties to handle -include
-			builder.setProperties(baseDir, properties.replaceHere(baseDir));
-		}
-		return pomFile;
-	}
-
-	private Optional<Xpp3Dom> getConfiguration(List<Plugin> plugins) {
-		return plugins.stream()
-			.filter(p -> Objects.equals(p, mojoExecution.getPlugin()))
-			.map(Plugin::getExecutions)
-			.flatMap(List::stream)
-			.filter(e -> Objects.equals(e.getId(), mojoExecution.getExecutionId()))
-			.findFirst()
-			.map(PluginExecution::getConfiguration)
-			.map(Xpp3Dom.class::cast)
-			.map(Xpp3Dom::new);
-	}
-
-	private Xpp3Dom defaultConfiguration() {
-		return new Xpp3Dom("configuration");
+			getExtension(artifact.getType()));
 	}
 
 	protected void reportErrorsAndWarnings(Builder builder) throws MojoFailureException {
@@ -781,28 +736,26 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 		}
 
 		Path path = directory.toPath();
+		Path meta_inf = Paths.get("META-INF");
 		try (Stream<Path> entries = Files.walk(path)) {
-			return !entries.filter(p -> !Files.isDirectory(p))
-				.filter(p -> !path.relativize(p)
-					.startsWith("META-INF/"))
-				.findFirst()
-				.isPresent();
+			return entries.allMatch(p -> Files.isDirectory(p) || path.relativize(p)
+				.startsWith(meta_inf));
 		} catch (IOException ioe) {
 			throw Exceptions.duck(ioe);
 		}
 	}
 
-	private void expandJar(Jar jar, File dir) throws Exception {
+	private void writeContent(Jar jar, File directory) throws Exception {
 		final long lastModified = jar.lastModified();
 		if (logger.isDebugEnabled()) {
 			logger.debug(String.format("Bundle lastModified: %tF %<tT.%<tL", lastModified));
 		}
-		dir = dir.getAbsoluteFile();
-		Files.createDirectories(dir.toPath());
+		directory = directory.getAbsoluteFile();
+		Files.createDirectories(directory.toPath());
 
 		for (Map.Entry<String, Resource> entry : jar.getResources()
 			.entrySet()) {
-			File outFile = IO.getBasedFile(dir, entry.getKey());
+			File outFile = IO.getBasedFile(directory, entry.getKey());
 			Resource resource = entry.getValue();
 			// Skip the copy if the source and target are the same file
 			if (resource instanceof FileResource) {
@@ -827,33 +780,42 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 				}
 			}
 		}
+	}
 
-		if (manifestOutOfDate() || getManifestPath().lastModified() < lastModified) {
+	private void writeManifest(Jar jar, File manifestPath) throws Exception {
+		final long lastModified = jar.lastModified();
+		if (outOfDate(manifestPath) || manifestPath.lastModified() < lastModified) {
 			if (logger.isDebugEnabled()) {
-				if (!manifestOutOfDate())
+				if (!outOfDate(manifestPath))
 					logger.debug(String.format("Updating lastModified: %tF %<tT.%<tL '%s'",
-						getManifestPath().lastModified(), getManifestPath()));
+						manifestPath.lastModified(), manifestPath));
 				else
-					logger.debug("Creating '{}'", getManifestPath());
+					logger.debug("Creating '{}'", manifestPath);
 			}
-			Files.createDirectories(getManifestPath().toPath()
+			Files.createDirectories(manifestPath.toPath()
 				.getParent());
-			try (OutputStream manifestOut = buildContext.newFileOutputStream(getManifestPath())) {
+			try (OutputStream manifestOut = buildContext.newFileOutputStream(manifestPath)) {
 				jar.writeManifest(manifestOut);
 			}
-			buildContext.setValue(MANIFEST_LAST_MODIFIED, getManifestPath().lastModified());
+			buildContext.setValue(LAST_MODIFIED, manifestPath.lastModified());
 		}
 	}
 
-	private boolean manifestOutOfDate() {
-		if (!getManifestPath().isFile()) {
+	private boolean outOfDate() {
+		String goal = mojoExecution.getMojoDescriptor()
+			.getGoal();
+		return outOfDate(isPackagingGoal(goal) ? getArtifactFile() : getManifestPath());
+	}
+
+	private boolean outOfDate(File target) {
+		if (!target.isFile()) {
 			return true;
 		}
 
-		long manifestLastModified = 0L;
-		if (buildContext.getValue(MANIFEST_LAST_MODIFIED) != null) {
-			manifestLastModified = (Long) buildContext.getValue(MANIFEST_LAST_MODIFIED);
+		long lastModified = 0L;
+		if (buildContext.getValue(LAST_MODIFIED) != null) {
+			lastModified = (Long) buildContext.getValue(LAST_MODIFIED);
 		}
-		return getManifestPath().lastModified() != manifestLastModified;
+		return target.lastModified() != lastModified;
 	}
 }
