@@ -9,6 +9,14 @@ import java.util.concurrent.Executors;
 import java.util.concurrent.ForkJoinPool;
 import java.util.concurrent.ScheduledExecutorService;
 
+import aQute.bnd.build.ProjectLauncher;
+import aQute.bnd.build.ProjectLauncher.LiveCoding;
+import aQute.bnd.maven.lib.configuration.Bndruns;
+import aQute.bnd.maven.lib.configuration.Bundles;
+import aQute.bnd.maven.lib.resolve.BndrunContainer;
+import aQute.bnd.maven.lib.resolve.Operation;
+import aQute.bnd.maven.lib.resolve.Scope;
+import aQute.bnd.unmodifiable.Sets;
 import org.apache.maven.execution.MavenSession;
 import org.apache.maven.plugin.AbstractMojo;
 import org.apache.maven.plugin.MojoExecutionException;
@@ -25,15 +33,6 @@ import org.eclipse.aether.RepositorySystemSession;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import aQute.bnd.build.ProjectLauncher;
-import aQute.bnd.build.ProjectLauncher.LiveCoding;
-import aQute.bnd.maven.lib.configuration.Bndruns;
-import aQute.bnd.maven.lib.configuration.Bundles;
-import aQute.bnd.maven.lib.resolve.BndrunContainer;
-import aQute.bnd.maven.lib.resolve.Operation;
-import aQute.bnd.maven.lib.resolve.Scope;
-import aQute.bnd.unmodifiable.Sets;
-
 @Mojo(name = "run", defaultPhase = LifecyclePhase.PACKAGE, requiresDirectInvocation = true, requiresProject = true, requiresDependencyResolution = ResolutionScope.TEST, threadSafe = true)
 public class RunMojo extends AbstractMojo {
 	private static final Logger									logger	= LoggerFactory.getLogger(RunMojo.class);
@@ -102,13 +101,8 @@ public class RunMojo extends AbstractMojo {
 
 			Operation operation = getOperation();
 
-			Bndruns bndruns = new Bndruns();
-			bndruns.addFile(bndrun);
-			List<File> bndrunFiles = bndruns.getFiles(project.getBasedir(), "*.bndrun");
-
-			File runFile = bndrunFiles.get(0);
-			logger.info("Running {}:", runFile);
-			errors += container.execute(runFile, "run", targetDir, operation);
+			logger.info("Running {}:", bndrun);
+			errors += container.execute(bndrun, "run", targetDir, operation);
 		} catch (Exception e) {
 			throw new MojoExecutionException(e.getMessage(), e);
 		}
